@@ -16,11 +16,13 @@
  * Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#ifndef RELAYD_CMD_2_2_H
-#define RELAYD_CMD_2_2_H
+#ifndef RELAYD_CMD_2_1_H
+#define RELAYD_CMD_2_1_H
 
 #include "lttng-relayd.h"
+#include "stream.h"
 
-int cmd_recv_stream_2_2(struct relay_command *cmd, struct relay_stream *stream);
+int cmd_recv_stream_2_1(struct relay_connection *conn,
+		struct relay_stream *stream);
 
-#endif /* RELAYD_CMD_2_2_H */
+#endif /* RELAYD_CMD_2_1_H */
