@@ -65,6 +65,7 @@ struct lttng_channel *channel_new_default_attr(int dom,
 		chan->attr.output = DEFAULT_KERNEL_CHANNEL_OUTPUT;
 		chan->attr.switch_timer_interval = DEFAULT_KERNEL_CHANNEL_SWITCH_TIMER;
 		chan->attr.read_timer_interval = DEFAULT_KERNEL_CHANNEL_READ_TIMER;
+		chan->attr.live_timer_interval = DEFAULT_KERNEL_CHANNEL_LIVE_TIMER;
 		break;
 	case LTTNG_DOMAIN_UST:
 		switch (type) {
@@ -76,6 +77,8 @@ struct lttng_channel *channel_new_default_attr(int dom,
 				DEFAULT_UST_UID_CHANNEL_SWITCH_TIMER;
 			chan->attr.read_timer_interval =
 				DEFAULT_UST_UID_CHANNEL_READ_TIMER;
+			chan->attr.live_timer_interval =
+				DEFAULT_UST_UID_CHANNEL_LIVE_TIMER;
 			break;
 		case LTTNG_BUFFER_PER_PID:
 		default:
@@ -86,6 +89,8 @@ struct lttng_channel *channel_new_default_attr(int dom,
 				DEFAULT_UST_PID_CHANNEL_SWITCH_TIMER;
 			chan->attr.read_timer_interval =
 				DEFAULT_UST_PID_CHANNEL_READ_TIMER;
+			chan->attr.live_timer_interval =
+				DEFAULT_UST_UID_CHANNEL_LIVE_TIMER;
 			break;
 		}
 		break;
@@ -361,7 +366,18 @@ int channel_ust_create(struct ltt_ust_session *usess,
 
 	/* Adding the channel to the channel hash table. */
 	rcu_read_lock();
-	lttng_ht_add_unique_str(usess->domain_global.channels, &uchan->node);
+	if (strncmp(uchan->name, DEFAULT_METADATA_NAME,
+				sizeof(uchan->name))) {
+		lttng_ht_add_unique_str(usess->domain_global.channels, &uchan->node);
+	} else {
+		/*
+		 * Copy channel attribute to session if this is metadata so if NO
+		 * application exists we can access that data in the shadow copy during
+		 * the global update of newly registered application.
+		 */
+		memcpy(&usess->metadata_attr, &uchan->attr,
+				sizeof(usess->metadata_attr));
+	}
 	rcu_read_unlock();
 
 	DBG2("Channel %s created successfully", uchan->name);
