@@ -26,7 +26,7 @@
 #include <common/defaults.h>
 
 #include "consumer.h"
-#include "health.h"
+#include "health-sessiond.h"
 #include "kernel-consumer.h"
 
 static char *create_channel_path(struct consumer_output *consumer,
@@ -105,7 +105,7 @@ int kernel_consumer_add_channel(struct consumer_socket *sock,
 		}
 	} else {
 		/* Empty path. */
-		pathname = "";
+		pathname = strdup("");
 	}
 
 	/* Prep channel message structure */
@@ -123,7 +123,8 @@ int kernel_consumer_add_channel(struct consumer_socket *sock,
 			CONSUMER_CHANNEL_TYPE_DATA,
 			channel->channel->attr.tracefile_size,
 			channel->channel->attr.tracefile_count,
-			monitor);
+			monitor,
+			channel->channel->attr.live_timer_interval);
 
 	health_code_update();
 
@@ -135,6 +136,7 @@ int kernel_consumer_add_channel(struct consumer_socket *sock,
 	health_code_update();
 
 error:
+	free(pathname);
 	return ret;
 }
 
@@ -167,7 +169,7 @@ int kernel_consumer_add_metadata(struct consumer_socket *sock,
 		}
 	} else {
 		/* Empty path. */
-		pathname = "";
+		pathname = strdup("");
 	}
 
 	/* Prep channel message structure */
@@ -184,7 +186,7 @@ int kernel_consumer_add_metadata(struct consumer_socket *sock,
 			DEFAULT_KERNEL_CHANNEL_OUTPUT,
 			CONSUMER_CHANNEL_TYPE_METADATA,
 			0, 0,
-			monitor);
+			monitor, 0);
 
 	health_code_update();
 
@@ -214,6 +216,7 @@ int kernel_consumer_add_metadata(struct consumer_socket *sock,
 	health_code_update();
 
 error:
+	free(pathname);
 	return ret;
 }
 
@@ -261,6 +264,40 @@ error:
 	return ret;
 }
 
+/*
+ * Sending the notification that all streams were sent with STREAMS_SENT.
+ */
+int kernel_consumer_streams_sent(struct consumer_socket *sock,
+		struct ltt_kernel_session *session, uint64_t channel_key)
+{
+	int ret;
+	struct lttcomm_consumer_msg lkm;
+	struct consumer_output *consumer;
+
+	assert(sock);
+	assert(session);
+
+	DBG("Sending streams_sent");
+	/* Get consumer output pointer */
+	consumer = session->consumer;
+
+	/* Prep stream consumer message */
+	consumer_init_streams_sent_comm_msg(&lkm,
+			LTTNG_CONSUMER_STREAMS_SENT,
+			channel_key, consumer->net_seq_index);
+
+	health_code_update();
+
+	/* Send stream and file descriptor */
+	ret = consumer_send_msg(sock, &lkm);
+	if (ret < 0) {
+		goto error;
+	}
+
+error:
+	return ret;
+}
+
 /*
  * Send all stream fds of kernel channel to the consumer.
  */
@@ -341,9 +378,6 @@ int kernel_consumer_send_session(struct consumer_socket *sock,
 		if (ret < 0) {
 			goto error;
 		}
-
-		/* Flag that at least the metadata has been sent to the consumer. */
-		session->consumer_fds_sent = 1;
 	}
 
 	/* Send channel and streams of it */
@@ -353,10 +387,21 @@ int kernel_consumer_send_session(struct consumer_socket *sock,
 		if (ret < 0) {
 			goto error;
 		}
+		if (monitor) {
+			/*
+			 * Inform the relay that all the streams for the
+			 * channel were sent.
+			 */
+			ret = kernel_consumer_streams_sent(sock, session, chan->fd);
+			if (ret < 0) {
+				goto error;
+			}
+		}
 	}
 
 	DBG("Kernel consumer FDs of metadata and channel streams sent");
 
+	session->consumer_fds_sent = 1;
 	return 0;
 
 error:
@@ -374,6 +419,7 @@ int kernel_consumer_destroy_channel(struct consumer_socket *socket,
 
 	DBG("Sending kernel consumer destroy channel key %d", channel->fd);
 
+	memset(&msg, 0, sizeof(msg));
 	msg.cmd_type = LTTNG_CONSUMER_DESTROY_CHANNEL;
 	msg.u.destroy_channel.key = channel->fd;
 
@@ -402,6 +448,7 @@ int kernel_consumer_destroy_metadata(struct consumer_socket *socket,
 
 	DBG("Sending kernel consumer destroy channel key %d", metadata->fd);
 
+	memset(&msg, 0, sizeof(msg));
 	msg.cmd_type = LTTNG_CONSUMER_DESTROY_CHANNEL;
 	msg.u.destroy_channel.key = metadata->fd;
 
