@@ -136,11 +136,11 @@ int kernel_create_channel(struct ltt_kernel_session *session,
 		goto error;
 	}
 
-	DBG3("Kernel create channel %s with attr: %d, %" PRIu64 ", %" PRIu64 ", %u, %u, %d",
+	DBG3("Kernel create channel %s with attr: %d, %" PRIu64 ", %" PRIu64 ", %u, %u, %d, %d",
 			chan->name, lkc->channel->attr.overwrite,
 			lkc->channel->attr.subbuf_size, lkc->channel->attr.num_subbuf,
 			lkc->channel->attr.switch_timer_interval, lkc->channel->attr.read_timer_interval,
-			lkc->channel->attr.output);
+			lkc->channel->attr.live_timer_interval, lkc->channel->attr.output);
 
 	/* Kernel tracer channel creation */
 	ret = kernctl_create_channel(session->fd, &lkc->channel->attr);
@@ -602,7 +602,7 @@ error:
  */
 ssize_t kernel_list_events(int tracer_fd, struct lttng_event **events)
 {
-	int fd, pos, ret;
+	int fd, ret;
 	char *event;
 	size_t nbmem, count = 0;
 	FILE *fp;
@@ -634,15 +634,15 @@ ssize_t kernel_list_events(int tracer_fd, struct lttng_event **events)
 		goto end;
 	}
 
-	while (fscanf(fp, "event { name = %m[^;]; };%n\n", &event, &pos) == 1) {
+	while (fscanf(fp, "event { name = %m[^;]; };\n", &event) == 1) {
 		if (count >= nbmem) {
 			struct lttng_event *new_elist;
+			size_t new_nbmem;
 
-			DBG("Reallocating event list from %zu to %zu bytes", nbmem,
-					nbmem * 2);
-			/* Double the size */
-			nbmem <<= 1;
-			new_elist = realloc(elist, nbmem * sizeof(struct lttng_event));
+			new_nbmem = nbmem << 1;
+			DBG("Reallocating event list from %zu to %zu bytes",
+					nbmem, new_nbmem);
+			new_elist = realloc(elist, new_nbmem * sizeof(struct lttng_event));
 			if (new_elist == NULL) {
 				PERROR("realloc list events");
 				free(event);
@@ -650,6 +650,10 @@ ssize_t kernel_list_events(int tracer_fd, struct lttng_event **events)
 				count = -ENOMEM;
 				goto end;
 			}
+			/* Zero the new memory */
+			memset(new_elist + nbmem, 0,
+				(new_nbmem - nbmem) * sizeof(struct lttng_event));
+			nbmem = new_nbmem;
 			elist = new_elist;
 		}
 		strncpy(elist[count].name, event, LTTNG_SYMBOL_NAME_LEN);
@@ -755,11 +759,11 @@ void kernel_destroy_session(struct ltt_kernel_session *ksess)
 	DBG("Tearing down kernel session");
 
 	/*
-	 * Destroy channels on the consumer if in no output mode because the
-	 * streams are in *no* monitor mode so we have to send a command to clean
-	 * them up or else they leaked.
+	 * Destroy channels on the consumer if at least one FD has been sent and we
+	 * are in no output mode because the streams are in *no* monitor mode so we
+	 * have to send a command to clean them up or else they leaked.
 	 */
-	if (!ksess->output_traces) {
+	if (!ksess->output_traces && ksess->consumer_fds_sent) {
 		int ret;
 		struct consumer_socket *socket;
 		struct lttng_ht_iter iter;
@@ -927,6 +931,8 @@ int kernel_snapshot_record(struct ltt_kernel_session *ksess,
 		(void) kernel_consumer_destroy_metadata(socket, ksess->metadata);
 	}
 
+	ret = LTTNG_OK;
+
 error_consumer:
 	/* Close newly opened metadata stream. It's now on the consumer side. */
 	err = close(ksess->metadata_stream_fd);
