@@ -28,7 +28,7 @@
 #include <common/defaults.h>
 
 #include "consumer.h"
-#include "health.h"
+#include "health-sessiond.h"
 #include "ust-consumer.h"
 #include "buffer-registry.h"
 #include "session.h"
@@ -102,7 +102,7 @@ static int ask_channel_creation(struct ust_app_session *ua_sess,
 		struct ust_app_channel *ua_chan, struct consumer_output *consumer,
 		struct consumer_socket *socket, struct ust_registry_session *registry)
 {
-	int ret;
+	int ret, output;
 	uint32_t chan_id;
 	uint64_t key, chan_reg_key;
 	char *pathname = NULL;
@@ -141,13 +141,21 @@ static int ask_channel_creation(struct ust_app_session *ua_sess,
 		chan_id = chan_reg->chan_id;
 	}
 
+	switch (ua_chan->attr.output) {
+	case LTTNG_UST_MMAP:
+	default:
+		output = LTTNG_EVENT_MMAP;
+		break;
+	}
+
 	consumer_init_ask_channel_comm_msg(&msg,
 			ua_chan->attr.subbuf_size,
 			ua_chan->attr.num_subbuf,
 			ua_chan->attr.overwrite,
 			ua_chan->attr.switch_timer_interval,
 			ua_chan->attr.read_timer_interval,
-			(int) ua_chan->attr.output,
+			ua_sess->live_timer_interval,
+			output,
 			(int) ua_chan->attr.type,
 			ua_sess->tracing_id,
 			pathname,
@@ -242,6 +250,7 @@ int ust_consumer_get_channel(struct consumer_socket *socket,
 	assert(ua_chan);
 	assert(socket);
 
+	memset(&msg, 0, sizeof(msg));
 	msg.cmd_type = LTTNG_CONSUMER_GET_CHANNEL;
 	msg.u.get_channel.key = ua_chan->key;
 
@@ -336,6 +345,7 @@ int ust_consumer_destroy_channel(struct consumer_socket *socket,
 	assert(ua_chan);
 	assert(socket);
 
+	memset(&msg, 0, sizeof(msg));
 	msg.cmd_type = LTTNG_CONSUMER_DESTROY_CHANNEL;
 	msg.u.destroy_channel.key = ua_chan->key;
 
@@ -461,6 +471,7 @@ int ust_consumer_metadata_request(struct consumer_socket *socket)
 			DBG("PID registry not found for session id %" PRIu64,
 					request.session_id_per_pid);
 
+			memset(&msg, 0, sizeof(msg));
 			msg.cmd_type = LTTNG_ERR_UND;
 			(void) consumer_send_msg(socket, &msg);
 			/*
