@@ -20,6 +20,10 @@
 
 #include <popt.h>
 
+#include <lttng/lttng.h>
+
+extern char *opt_relayd_path;
+
 struct cmd_struct;
 
 char *get_session_name(void);
@@ -44,4 +48,15 @@ int get_count_order_u64(uint64_t x);
  */
 int get_count_order_ulong(unsigned long x);
 
+const char *get_domain_str(enum lttng_domain_type domain);
+
+static inline
+void print_missing_domain(void)
+{
+	ERR("Please specify a domain (-k/-u/-j).");
+}
+
+int spawn_relayd(const char *pathname, int port);
+int check_relayd(void);
+
 #endif /* _LTTNG_UTILS_H */
