@@ -51,6 +51,11 @@ static int resize_poll_event(struct lttng_poll_event *events,
 		PERROR("realloc epoll add");
 		goto error;
 	}
+	if (new_size > events->alloc_size) {
+		/* Zero newly allocated memory */
+		memset(ptr + events->alloc_size, 0,
+			(new_size - events->alloc_size) * sizeof(*ptr));
+	}
 	events->events = ptr;
 	events->alloc_size = new_size;
 
@@ -119,6 +124,11 @@ int compat_epoll_add(struct lttng_poll_event *events, int fd, uint32_t req_event
 		goto error;
 	}
 
+	/*
+	 * Zero struct epoll_event to ensure all representations of its
+	 * union are zeroed.
+	 */
+	memset(&ev, 0, sizeof(ev));
 	ev.events = req_events;
 	ev.data.fd = fd;
 
@@ -250,6 +260,7 @@ error:
 void compat_epoll_set_max_size(void)
 {
 	int ret, fd;
+	ssize_t size_ret;
 	char buf[64];
 
 	poll_max_size = DEFAULT_POLL_SIZE;
@@ -259,11 +270,16 @@ void compat_epoll_set_max_size(void)
 		return;
 	}
 
-	ret = read(fd, buf, sizeof(buf));
-	if (ret < 0) {
+	size_ret = lttng_read(fd, buf, sizeof(buf));
+	/*
+	 * Allow reading a file smaller than buf, but keep space for
+	 * final \0.
+	 */
+	if (size_ret < 0 || size_ret >= sizeof(buf)) {
 		PERROR("read set max size");
 		goto error;
 	}
+	buf[size_ret] = '\0';
 
 	poll_max_size = atoi(buf);
 	if (poll_max_size == 0) {
